@@ -55,6 +55,7 @@
 #include "control.h"
 #include "sdpd.h"
 #include "glib-helper.h"
+#include "dbus-common.h"
 
 #define AVCTP_PSM 23
 
@@ -97,11 +98,7 @@
 
 static DBusConnection *connection = NULL;
 
-static uint32_t tg_record_id = 0;
-static uint32_t ct_record_id = 0;
-
-static GIOChannel *avctp_server = NULL;
-
+static GSList *servers = NULL;
 static GSList *sessions = NULL;
 
 typedef enum {
@@ -150,6 +147,13 @@ struct avrcp_header {
 #error "Unknown byte order"
 #endif
 
+struct avctp_server {
+	bdaddr_t src;
+	GIOChannel *io;
+	uint32_t tg_record_id;
+	uint32_t ct_record_id;
+};
+
 struct avctp {
 	struct audio_device *dev;
 
@@ -414,12 +418,20 @@ static void avctp_unref(struct avctp *session)
 {
 	sessions = g_slist_remove(sessions, session);
 
-	if (session->state == AVCTP_STATE_CONNECTED)
+	if (session->state == AVCTP_STATE_CONNECTED) {
+		gboolean value = FALSE;
 		g_dbus_emit_signal(session->dev->conn,
 						session->dev->path,
 						AUDIO_CONTROL_INTERFACE,
 						"Disconnected",
 						DBUS_TYPE_INVALID);
+		dbus_connection_emit_property_changed(session->dev->conn,
+						session->dev->path,
+						AUDIO_CONTROL_INTERFACE,
+						"Connected",
+						DBUS_TYPE_BOOLEAN, &value);
+	}
+
 	if (session->sock >= 0)
 		close(session->sock);
 	if (session->io)
@@ -597,6 +609,7 @@ static void init_uinput(struct avctp *session)
 static void avctp_connect_session(struct avctp *session)
 {
 	GIOChannel *io;
+	gboolean value;
 
 	session->state = AVCTP_STATE_CONNECTED;
 	session->dev = manager_find_device(&session->dst, NULL, FALSE);
@@ -607,9 +620,15 @@ static void avctp_connect_session(struct avctp *session)
 
 	init_uinput(session);
 
+	value = TRUE;
 	g_dbus_emit_signal(session->dev->conn, session->dev->path,
 					AUDIO_CONTROL_INTERFACE, "Connected",
 					DBUS_TYPE_INVALID);
+	dbus_connection_emit_property_changed(session->dev->conn,
+					session->dev->path,
+					AUDIO_CONTROL_INTERFACE,
+					"Connected",
+					DBUS_TYPE_BOOLEAN, &value);
 
 	if (session->io)
 		g_source_remove(session->io);
@@ -627,10 +646,6 @@ static void auth_cb(DBusError *derr, void *user_data)
 
 	if (derr && dbus_error_is_set(derr)) {
 		error("Access denied: %s", derr->message);
-		if (dbus_error_has_name(derr, DBUS_ERROR_NO_REPLY)) {
-			debug("Canceling authorization request");
-			btd_cancel_authorization(&session->src, &session->dst);
-		}
 
 		avctp_unref(session);
 		return;
@@ -703,7 +718,7 @@ drop:
 	close(session->sock);
 }
 
-static GIOChannel *avctp_server_socket(gboolean master)
+static GIOChannel *avctp_server_socket(const bdaddr_t *src, gboolean master)
 {
 	GIOChannel *io;
 	int lm = L2CAP_LM_AUTH | L2CAP_LM_ENCRYPT;
@@ -711,7 +726,7 @@ static GIOChannel *avctp_server_socket(gboolean master)
 	if (master)
 		lm |= L2CAP_LM_MASTER;
 
-	io = bt_l2cap_listen(BDADDR_ANY, AVCTP_PSM, 0, lm, avctp_server_cb,
+	io = bt_l2cap_listen(src, AVCTP_PSM, 0, lm, avctp_server_cb,
 				NULL);
 	if (!io) {
 		error("Unable to allocate new io channel");
@@ -810,14 +825,12 @@ void avrcp_disconnect(struct audio_device *dev)
 	control->session = NULL;
 }
 
-int avrcp_init(DBusConnection *conn, GKeyFile *config)
+int avrcp_register(DBusConnection *conn, const bdaddr_t *src, GKeyFile *config)
 {
 	sdp_record_t *record;
 	gboolean tmp, master = TRUE;
 	GError *err = NULL;
-
-	if (avctp_server)
-		return 0;
+	struct avctp_server *server;
 
 	if (config) {
 		tmp = g_key_file_get_boolean(config, "General",
@@ -829,7 +842,12 @@ int avrcp_init(DBusConnection *conn, GKeyFile *config)
 			master = tmp;
 	}
 
-	connection = dbus_connection_ref(conn);
+	server = g_new0(struct avctp_server, 1);
+	if (!server)
+		return -ENOMEM;
+
+	if (!connection)
+		connection = dbus_connection_ref(conn);
 
 	record = avrcp_tg_record();
 	if (!record) {
@@ -837,12 +855,12 @@ int avrcp_init(DBusConnection *conn, GKeyFile *config)
 		return -1;
 	}
 
-	if (add_record_to_server(BDADDR_ANY, record) < 0) {
+	if (add_record_to_server(src, record) < 0) {
 		error("Unable to register AVRCP target service record");
 		sdp_record_free(record);
 		return -1;
 	}
-	tg_record_id = record->handle;
+	server->tg_record_id = record->handle;
 
 	record = avrcp_ct_record();
 	if (!record) {
@@ -850,34 +868,61 @@ int avrcp_init(DBusConnection *conn, GKeyFile *config)
 		return -1;
 	}
 
-	if (add_record_to_server(BDADDR_ANY, record) < 0) {
+	if (add_record_to_server(src, record) < 0) {
 		error("Unable to register AVRCP controller service record");
 		sdp_record_free(record);
 		return -1;
 	}
-	ct_record_id = record->handle;
+	server->ct_record_id = record->handle;
 
-	avctp_server = avctp_server_socket(master);
-	if (!avctp_server)
+	server->io = avctp_server_socket(src, master);
+	if (!server->io) {
+		remove_record_from_server(server->ct_record_id);
+		remove_record_from_server(server->tg_record_id);
+		g_free(server);
 		return -1;
+	}
+
+	bacpy(&server->src, src);
+
+	servers = g_slist_append(servers, server);
 
 	return 0;
 }
 
-void avrcp_exit(void)
+static struct avctp_server *find_server(GSList *list, const bdaddr_t *src)
 {
-	if (!avctp_server)
+	GSList *l;
+
+	for (l = list; l; l = l->next) {
+		struct avctp_server *server = l->data;
+
+		if (bacmp(&server->src, src) == 0)
+			return server;
+	}
+
+	return NULL;
+}
+
+void avrcp_unregister(const bdaddr_t *src)
+{
+	struct avctp_server *server;
+
+	server = find_server(servers, src);
+	if (!server)
 		return;
 
-	g_io_channel_close(avctp_server);
-	g_io_channel_unref(avctp_server);
-	avctp_server = NULL;
+	servers = g_slist_remove(servers, server);
 
-	remove_record_from_server(ct_record_id);
-	ct_record_id = 0;
+	remove_record_from_server(server->ct_record_id);
+	remove_record_from_server(server->tg_record_id);
 
-	remove_record_from_server(tg_record_id);
-	tg_record_id = 0;
+	g_io_channel_close(server->io);
+	g_io_channel_unref(server->io);
+	g_free(server);
+
+	if (servers)
+		return;
 
 	dbus_connection_unref(connection);
 	connection = NULL;
@@ -904,14 +949,47 @@ static DBusMessage *control_is_connected(DBusConnection *conn,
 	return reply;
 }
 
+static DBusMessage *control_get_properties(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	struct audio_device *device = data;
+	DBusMessage *reply;
+	DBusMessageIter iter;
+	DBusMessageIter dict;
+	gboolean value;
+
+	reply = dbus_message_new_method_return(msg);
+	if (!reply)
+		return NULL;
+
+	dbus_message_iter_init_append(reply, &iter);
+
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
+			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+
+	/* Connected */
+	value = (device->control->session != NULL);
+	dbus_message_iter_append_dict_entry(&dict, "Connected",
+						DBUS_TYPE_BOOLEAN, &value);
+
+	dbus_message_iter_close_container(&iter, &dict);
+
+	return reply;
+}
+
 static GDBusMethodTable control_methods[] = {
-	{ "IsConnected",	"",	"b",	control_is_connected },
+	{ "IsConnected",	"",	"b",	control_is_connected,
+						G_DBUS_METHOD_FLAG_DEPRECATED },
+	{ "GetProperties",	"",	"a{sv}",control_get_properties },
 	{ NULL, NULL, NULL, NULL }
 };
 
 static GDBusSignalTable control_signals[] = {
-	{ "Connected",			""	},
-	{ "Disconnected",		""	},
+	{ "Connected",			"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
+	{ "Disconnected",		"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
+	{ "PropertyChanged",		"sv"	},
 	{ NULL, NULL }
 };
 
