@@ -22,10 +22,6 @@
  *
  */
 
-#define MAX_PATH_LENGTH 64 /* D-Bus path */
-#define AUDIO_MANAGER_PATH "/org/bluez/audio"
-#define AUDIO_MANAGER_INTERFACE "org.bluez.audio.Manager"
-
 struct enabled_interfaces {
 	gboolean headset;
 	gboolean gateway;
@@ -39,7 +35,7 @@ typedef void (*create_dev_cb_t) (struct audio_device *dev, void *user_data);
 int audio_manager_init(DBusConnection *conn, GKeyFile *config);
 void audio_manager_exit(void);
 
-gboolean server_is_enabled(uint16_t svc);
+gboolean server_is_enabled(bdaddr_t *src, uint16_t svc);
 
 struct audio_device *manager_find_device(const bdaddr_t *bda, const char *interface,
 					gboolean connected);
